#ifndef INC_REQ_CHC
#define INC_REQ_CHC
#include "CHC.hh"
#include <math.h>

skeleton CHC
{

	// Problem ---------------------------------------------------------------

	Problem::Problem ():_numvar(0),_numclause(0),_clauses(NULL),_lenclause(0)
	{}

	ostream& operator<< (ostream& os, const Problem& pbm)
	{
		os << endl << endl << "Number of variables " << pbm._numvar
           << endl << endl << "Number of clauses " << pbm._numclause
           << endl << endl << "Length of clauses " << pbm._lenclause
		   << endl << endl
		   << " Clauses: " << endl;

		for (int i=0;i<pbm._numclause;i++)
        {
            os << "\t clause " << i << "\t ";
            for(int j = 0; j < pbm._lenclause; j++)
			    os << pbm._clauses[i][j] << " ";
		    os << endl;
        }
		return os;
	}

	istream& operator>> (istream& is, Problem& pbm)
	{
        int l;
        int n;
		is >> pbm._numvar >> pbm._numclause >> pbm._lenclause;

        n = pbm._lenclause;
		// read clauses
		pbm._clauses = new int*[pbm._numclause];

		for (int i = 0; i < pbm._numclause; i++)
		{
            pbm._clauses[i] = new int[n];
			for(int j = 0; j < n;j++)
            {
				is >> l;
                pbm._clauses[i][j] = l;
            }
            is >> l;
        }

		return is;
	}

	Problem& Problem::operator=  (const Problem& pbm)
	{
        int n;
		_numvar=pbm.numvar();
        for(int i = 0; i < _numclause;i++)
            delete [] _clauses[i];
        delete [] _clauses;

        _numclause = pbm.numclause();
        n = _lenclause = pbm.lenclause();

        _clauses = new int *[_numclause];

        for(int i = 0; i < pbm._numclause;i++)
        {
            _clauses[i] = new int [n];
            for(int j = 0; j < n ; j++)
    	        _clauses[i][j] = pbm._clauses[i][j];
        }
		return *this;
	}

	bool Problem::operator== (const Problem& pbm) const
	{
		if (_numvar!=pbm.numvar()) return false;
		for (int i = 0; i < _numclause; i++)
            for(int j = 0; j < _lenclause;j++)
			if ( _clauses[i][j] != pbm._clauses[i][j])
				return false;
		return true;
	}

	bool Problem::operator!= (const Problem& pbm) const
	{
		return !(*this == pbm);
	}

	Direction Problem::direction() const
	{
		return maximize;
		//return minimize;
	}

	int Problem::numvar() const
	{
		return _numvar;
	}

	int Problem::numclause() const
	{
		return _numclause;
	}

	int Problem::lenclause() const
	{
		return _lenclause;
	}

	int *Problem::clause(const int i) const
	{
		return _clauses[i];
	}

	Problem::~Problem()
	{
        for(int i = 0;i < _numclause;i++)
            delete [] _clauses;

        delete [] _clauses;
	}

	// Solution --------------------------------------------------------------

	Solution::Solution (const Problem& pbm):_pbm(pbm),_var(pbm.numvar())
	{}


	const Problem& Solution::pbm() const
	{
		return _pbm;
	}

	Solution::Solution(const Solution& sol):_pbm(sol.pbm())
	{
		*this=sol;
	}


	istream& operator>> (istream& is, Solution& sol)
	{
		for (int i=0;i<sol.pbm().numvar();i++)
			is >> sol._var[i];
		return is;
	}

	ostream& operator<< (ostream& os, const Solution& sol)
	{
		for (int i=0;i<sol.pbm().numvar();i++)
			os << " " << sol._var[i];
		return os;
	}

	NetStream& operator << (NetStream& ns, const Solution& sol)
	{
		for (int i=0;i<sol._var.size();i++)
			ns << sol._var[i];
		return ns;
	}


	NetStream& operator >> (NetStream& ns, Solution& sol)
	{
		for (int i=0;i<sol._var.size();i++)
			ns >> sol._var[i];
		return ns;
	}


 	Solution& Solution::operator= (const Solution &sol)
	{
		_var = sol._var;
		return *this;
	}


	bool Solution::operator== (const Solution& sol) const
	{
		if (sol.pbm() != _pbm) return false;
		return true;
	}

	bool Solution::operator!= (const Solution& sol) const
	{
		return !(*this == sol);
	}

	void Solution::initialize()
	{
		for (int i=0;i<_pbm.numvar();i++)
			_var[i]=rand_int(0,1);
	}


	double Solution::fitness () const
	{
		double fitness = 0.0;
        int acum = 0;
	
		if(_var[0] == 2) return 0.0;
		
        for(int i = 0; i < _pbm.numclause(); i++)
        {
            int *rl = _pbm.clause(i);
            acum = 0;
            for(int j = 0; (j < _pbm.lenclause()) && (acum != 1);j++)
            {
                if( ((rl[j] < 0) && (_var[(int)abs(rl[j])-1] == 0))
				 || ((rl[j] > 0) && (_var[rl[j]-1] == 1)) )
				 	acum = 1;
            }
			fitness += acum;
        }
		return fitness;
	}


	char *Solution::to_String() const
	{
		return (char *)_var.get_first();
	}

	void Solution::to_Solution(char *_string_)
	{
		int *ptr=(int *)_string_;
		for (int i=0;i<_pbm.numvar();i++)
		{
			_var[i]=*ptr;
			ptr++;
		}
	}
	
	unsigned int Solution::size() const
	{
		return (_pbm.numvar() * sizeof(int));
	}

	int Solution::lengthInBits() const
	{
		return _pbm.numvar();
	}

	void Solution::flip(const int index)
	{
		_var[index] = 1 - _var[index];
	}

	bool Solution::equalb(const int index,Solution &s)
	{
		return _var[index] == s._var[index];
	}

	void Solution::swap(const int index, Solution &s)
	{
		int aux = s._var[index];
		s._var[index] = _var[index];
		_var[index] = aux;
	}

	void Solution::invalid()
	{
		_var[0] = 2;
	}

	int& Solution::var(const int index)
	{
		return _var[index];
	}


	Rarray<int>& Solution::array_var()
	{
		return _var;
	}

	Solution::~Solution()
	{}

	// UserStatistics -------------------------------------------------------

	UserStatistics::UserStatistics ()
	{}

	ostream& operator<< (ostream& os, const UserStatistics& userstat)
	{
		os << "\n---------------------------------------------------------------" << endl;
		os << "                   STATISTICS OF TRIALS                   	 " << endl;
		os << "------------------------------------------------------------------" << endl;

		for (int i=0;i< userstat.result_trials.size();i++)
		{
			os << endl
			   << userstat.result_trials[i].trial
			   << "\t" << userstat.result_trials[i].best_cost_trial
			   << "\t\t" << userstat.result_trials[i].worst_cost_trial
			   << "\t\t\t" << userstat.result_trials[i].nb_evaluation_best_found_trial
			   << "\t\t\t" << userstat.result_trials[i].time_best_found_trial
			   << "\t\t" << userstat.result_trials[i].time_spent_trial;
		}
		os << endl << "------------------------------------------------------------------" << endl;
		return os;
	}

	UserStatistics& UserStatistics::operator= (const UserStatistics& userstats)
	{
		result_trials=userstats.result_trials;
		return (*this);
	}

	void UserStatistics::update(const Solver& solver)
	{
		if (!(solver.pid()==0 && (solver.end_trial()==true)
		  && (solver.current_iteration()==solver.setup().nb_evolution_steps())))
			return;

		struct user_stat *new_stat;

		if ((new_stat=(struct user_stat *)malloc(sizeof(struct user_stat)))==NULL)
			show_message(7);
		new_stat->trial         		 = solver.current_trial();
		new_stat->nb_evaluation_best_found_trial = solver.iteration_best_found_in_trial();
		new_stat->worst_cost_trial     		 = solver.worst_cost_trial();
		new_stat->best_cost_trial     		 = solver.best_cost_trial();
		new_stat->time_best_found_trial		 = solver.time_best_found_trial();
		new_stat->time_spent_trial 		 = solver.time_spent_trial();

		result_trials.append(*new_stat);
	}

	void UserStatistics::clear()
	{
		result_trials.remove();
	}

	UserStatistics::~UserStatistics()
	{
		result_trials.remove();
	}

	//  User_Operator:Intra_operator ---------------------------------------------------------

	User_Operator::User_Operator(const unsigned int _number_op):Intra_Operator(_number_op)
	{}

	void User_Operator::execute(Rarray<Solution*>& sols) const
	{}

	void User_Operator::setup(char line[MAX_BUFFER])
	{}

	Intra_Operator *User_Operator::create(const unsigned int _number_op)
	{
		return new User_Operator(_number_op);
	}

	ostream& operator<< (ostream& os, const User_Operator&  u_op)
	{
		 os << "User Operator.";
		 return os;
	}

	void User_Operator::RefreshState(const StateCenter& _sc) const
	{}

	void User_Operator::UpdateFromState(const StateCenter& _sc)
	{}

	User_Operator::~User_Operator()
	{}


// StopCondition_1 -------------------------------------------------------------------------------------

	StopCondition_1::StopCondition_1():StopCondition()
	{}

	bool StopCondition_1::EvaluateCondition(const Problem& pbm,const Solver& solver,const SetUpParams& setup)
	{
		return ((solver.current_trial()==setup.independent_runs()) && (solver.current_iteration()==setup.nb_evolution_steps()));
	}

	StopCondition_1::~StopCondition_1()
	{}


	//------------------------------------------------------------------------
	// Specific methods ------------------------------------------------------
	//------------------------------------------------------------------------

	bool terminateQ (const Problem& pbm, const Solver& solver,
			 const SetUpParams& setup)
	{
		StopCondition_1 stop;
		return stop.EvaluateCondition(pbm,solver,setup);
	}
}
#endif

